@@ -15,6 +15,7 @@ import time
 import functools
 import math
 import os
+import socket
 import stat
 import string
 import logging
@@ -22,8 +23,13 @@ import threading
 import io
 from collections import defaultdict
 
+from botocore.exceptions import IncompleteReadError
+from botocore.exceptions import ReadTimeoutError
+
+from s3transfer.compat import SOCKET_ERROR
 from s3transfer.compat import rename_file
 from s3transfer.compat import seekable
+from s3transfer.compat import fallocate
 
 
 MAX_PARTS = 10000
@@ -35,6 +41,11 @@ MIN_UPLOAD_CHUNKSIZE = 5 * (1024 ** 2)
 logger = logging.getLogger(__name__)
 
 
+S3_RETRYABLE_DOWNLOAD_ERRORS = (
+    socket.timeout, SOCKET_ERROR, ReadTimeoutError, IncompleteReadError
+)
+
+
 def random_file_extension(num_digits=8):
     return ''.join(random.choice(string.hexdigits) for _ in range(num_digits))
 
@@ -51,6 +62,10 @@ def signal_transferring(request, operation_name, **kwargs):
         request.body.signal_transferring()
 
 
+def calculate_num_parts(size, part_size):
+    return int(math.ceil(size / float(part_size)))
+
+
 def calculate_range_parameter(part_size, part_index, num_parts,
                               total_size=None):
     """Calculate the range parameter for multipart downloads/copies
@@ -126,6 +141,24 @@ def invoke_progress_callbacks(callbacks, bytes_transferred):
             callback(bytes_transferred=bytes_transferred)
 
 
+def get_filtered_dict(original_dict, whitelisted_keys):
+    """Gets a dictionary filtered by whitelisted keys
+
+    :param original_dict: The original dictionary of arguments to source keys
+        and values.
+    :param whitelisted_key: A list of keys to include in the filtered
+        dictionary.
+
+    :returns: A dictionary containing key/values from the original dictionary
+        whose key was included in the whitelist
+    """
+    filtered_dict = {}
+    for key, value in original_dict.items():
+        if key in whitelisted_keys:
+            filtered_dict[key] = value
+    return filtered_dict
+
+
 class CallArgs(object):
     def __init__(self, **kwargs):
         """A class that records call arguments
@@ -266,6 +299,17 @@ class OSUtils(object):
             return True
         return False
 
+    def get_temp_filename(self, filename):
+        return filename + os.extsep + random_file_extension()
+
+    def allocate(self, filename, size):
+        try:
+            with self.open(filename, 'wb') as f:
+                fallocate(f, size)
+        except (OSError, IOError):
+            self.remove_file(filename)
+            raise
+
 
 class DeferredOpenFile(object):
     def __init__(self, filename, start_byte=0, mode='rb', open_function=open):
